ppm1 <- function(fmla,Data,qtype=c("linear","quadratic"),
                 stype=1, ndum=NULL,ntot=NULL,maxit=50,preponly=FALSE,...) { 
#
# Function ppm1 --- to fit a Poisson log linear model to a
# one-dimensional point process.
#

if(is.null(ndum) & is.null(ntot))
	stop("One of \"ndum\" or \"ntot\" must be specified.\n")

# Extract the process from the formula.
lf <- as.character(fmla)
xnm <- lf[2]
if(exists(xnm,envir=environment(fmla))) {
	x <- get(xnm,envir=environment(fmla))
} else if(exists(xnm,envir=globalenv())) {
	x <- get(xnm,envir=globalenv())
} else {
	stop(paste("Object",xnm,"not found.\n"))
}
interval <- attr(x,"interval")
tx <- x
nint <- length(x) - 1
T0 <- interval[1]
if(T0 < min(x)) {
	tx <- c(T0,tx) 
	nint <- nint + 1
}
T1 <- interval[2]
if(T1 > max(x)) {
	tx <- c(tx,T1)
	nint <- nint + 1
}

qtype <- match.arg(qtype)

# Form the dummy points according to "stype". NOTE:
# stype = 1 <--> ntot points equispaced on [T0,T1]
# stype = 2 <--> ndum points equispaced on each (t_i,t_{i+1}),
# stype = 3 <--> approx. ntot points; number of points in each
#                interval (t_i,t_{i+1}) approximately
#                proportional to the interval length; points
#                equispaced on each (t_i,t_{i+1}),
switch(EXPR=stype,
	{
		if(is.null(ntot))
			stop("When \"stype\" is 1, \"\ntot\" must be specified.\n")
		if(qtype=="quadratic")
			stop("For quadratic weights \"stype\" must be 2 or 3.\n")
		dummy <- seq(T0,T1,length=ntot+2)
	},
	{
		if(is.null(ndum))
			stop("When \"stype\" is 2, \"\ndum\" must be specified.\n")
		if(ndum%%2 == 0 & qtype=="quadratic")
			stop(paste("Number of dummy points per interval must be\n",
                                   "odd when using quadratic quadrature.\n"))
		dummy  <- lapply(1:nint,function(i){
		   seq(tx[i],tx[i+1],length=ndum+2)[-c(1,ndum+2)]
		})
		dummy <- c(T0,unlist(dummy),T1)
	},
	{
		if(is.null(ntot)) {
			if(is.null(ndum))
			   stop(paste("When \"stype\" is 3, one of \"ntot\" or",
                                      "\"\ndum\" must be specified.\n"))
			ntot <- ndum*nint
			use.ndum <- TRUE
		} else use.ndum <- FALSE
		xxx <- seq(T0,T1,length=ntot+2)[-c(1,ntot+2)]
		yyy <- cut(xxx,breaks=tx)
		tbl <- table(yyy)
		iii <- which(tbl%%2 == 0)
		tbl[iii] <- tbl[iii] + (-1)^(seq(along=iii))
		if(any(tbl<0)) {
			iii <- which(tbl < 0)
			for(i in iii) {
				tbl[i] <- tbl[i] + 2
				imax <- which.max(tbl)
				tbl[imax] <- tbl[imax] - 2
			}
		}
		if(any(tbl<0))
			stop(paste("Can't seem to effect \"stype = 3\".\n",
			"Try increasing",if(use.ndum) "\"ndum\"" else "\"ntot\"",
                        "or using", "\"stype = 2\".\n"))
		dummy  <- lapply(1:nint,function(i){
		   seq(tx[i],tx[i+1],length=tbl[i]+2)[-c(1,tbl[i]+2)]
		})
		dummy <- c(T0,unlist(dummy),T1)
	}
)

dummy <- as.numeric(setdiff(sprintf("%f",dummy),sprintf("%f",x)))

# Form the quadrature points and the indicators.
Ndat    <- length(x)
Ndum    <- length(dummy)
qpoints <- c(x,dummy)
indic   <- c(rep(1,Ndat),rep(0,Ndum))
o       <- order(qpoints)
qpoints <- qpoints[o]
indic   <- indic[o]

# Form the weights.
delta <- diff(qpoints)
switch(EXPR=qtype,
	linear = {
		w <- 0.5*(c(0,delta) + c(delta,0))
	},
	quadratic = {
		w <- (c(0,delta) + c(delta,0))*
                      rep(1:2,length=length(qpoints))/3
	},
	stop(paste("Quadrature type",qtype,"not recognized.\n"))
)

# Form the response.
z <- indic/w

# If preponly return the structure created so far.
if(preponly) return(data.frame(qpoints=qpoints,indic=indic,z=z,w=w))

# Fit the model.
rhs  <- rev(as.character(fmla))[1]
fmla <- as.formula(paste("z ~",rhs))
environment(fmla) <- environment()
Dat <- prepDat(qpoints,Data,interval=interval,...)
fit <- glm(fmla,family=quasi(link=log,variance=mu),
                   weights=w,data=Dat,maxit=maxit)

# Extras.
maxll <- -(deviance(fit)/2 + sum(log(w[indic==1])) + Ndat)
aic   <- -2*maxll + 2*(length(Data)+1)
fv    <- predict(fit,type="response")
iFn   <- splinefun(x=qpoints,y=fv)
attr(iFn,"ylim") <- range(fv)

attr(fit,"log.likelihood") <- maxll
attr(fit,"aic") <- aic
attr(fit,"iFn") <- iFn
attr(fit,"Cov") <- vcov(fit,dispersion=1)
class(fit) <- c("ppm1",class(fit))
fit
}
