\name{ppm1}
\alias{ppm1}
\title{
    One dimensional point process modelling.
}
\description{
    Fits an exponential family model, for the intensity of a one
    dimensional (Poisson) process, to an observed one dimensional
    point pattern.
}
\usage{
    ppm1(fmla, Data, qtype = c("linear", "quadratic"),
    stype = 1, ndum = NULL, ntot = NULL, maxit = 50,
    preponly = FALSE, ...)
}
\arguments{
  \item{fmla}{
  A formula of the form \code{X ~ <predictors>} where \code{X}
  is a one dimensional point pattern, i.e. a realization of a one
  dimensional point process.  See examples.
}
  \item{Data}{
  An object specifying the predictors in the model for the intensity.
  It should be a list whose entries are either functions (to be
  evaluated at a set of quadrature points) or vectors (either
  numeric or factor) of length equal to the length of a vector of
  quadrature points.
}
  \item{qtype}{
   Text string specifying the quadrature type, either \dQuote{quadratic}
   (effectively implying the use of Simpson's rule) or \dQuote{linear}
   (effectively implying the use of the trapezoid rule).  May be
   abbreviated to \dQuote{"q"} or \dQuote{"l"}.
}
  \item{stype}{
   Integer scalar, either 1, 2, or 3, specifying the \dQuote{spacing
   type} for the dummy points.  If \code{stype == 1} then the dummy
   points consist of \code{ntot} points equispaced on \eqn{[T0,T1]},
   the interval over which the pattern is observed.  If \code{stype
   == 2} then the dummy points consist of \code{ndum} points
   equispaced on each \eqn{(t_i,t_{i+1})} where the \eqn{t_i} are
   the observed events of the pattern. If \code{stype == 2} and
   \code{qtype == "q"}\ then an error is given unless \code{ndum}
   is \emph{odd}.  If \code{stype == 3} then the dummy points
   consist of \emph{approximately} \code{ntot} points.  The number
   of points in each interval \eqn{(t_i,t_{i+1})} is approximately
   proportional to the interval length.  The points in each such
   interval are equispaced in that interval.  If \code{qtype=="q"}
   an odd number of dummy points will be placed in each interval.
}
  \item{ndum}{
  If \code{stype == 2} this argument gives the number of dummy
  points to be placed (equispaced) in each interval between
  successive events of the pattern.  In this case if
  \code{qtype=="q"} then \code{ndum} must be odd, otherwise an error
  is given.  If \code{stype == 3} and if \code{ntot} (see below)
  is not specified then this argument is (roughly) the mean number
  of dummy points to be placed (equispaced) in each interval between
  successive events.  (I.e. in this situation \code{ntot} is taken to
  be \code{ndum} times the number of intervals.)  If \code{stype ==
  1} or if \code{stype == 3} and \code{ntot} \emph{is} specified,
  then \code{ndum} is ignored.
}
  \item{ntot}{
  The total (or approximate total) number of dummy points to be created.
  This argument is ignored if \code{stype == 2}.
}
  \item{maxit}{
  The maximum number of iterations used in the iteratively reweighted
  least squares algorithm used by \code{\link{glm}()}.
}
  \item{preponly}{
  Logical scaler, if \code{TRUE} then this function only does \dQuote{data
  preparation}.  In this case this function returns a data frame of \dQuote{prepared
  data}.  It may be necessary (or at least convenient) to use \code{preponly=TRUE}
  when there is a requirement to construct vector predictors as part of the
  \code{Data} argument.
}
  \item{\dots}{
  Auxilliary arguments to be passed to the functions forming components
  of the list \code{Data}. 
}
}
\section{Note}{
One of \code{ndum} or \code{ntot} must always be specified.
}
\value{
   If \code{preponly=FALSE}, an object of class \code{ppm1}, also
   inheriting from class \code{glm}.  This object is essentially the
   object returned by \code{glm()}, as applied via the Berman-Turner
   device, with four extra attributes:
   \itemize{
   \item \code{log.likelihood}.  The approximate maximal log likelihood
   of the model.
   \item \code{aic}.  The Akaike information criterion for the fit.
   \item \code{Cov}.  The estimated covariance matrix of the parameter
   estimates
   \item \code{iFn}.  A function (an object of class \code{splinefun})
   specifying the fitted intensity function.
   }

   If \code{preponly=TRUE}, a data frame with columns \code{qpoints}
   (the quadrature points, data and dummy), \code{indic} (an
   indicator variable equal to 1 where the corresponding entry
   of \code{qpoints} is a data point, and equal to 0 where the
   corresponding entry of \code{qpoints} is a dummy point, \code{z = indic/w},
   and \code{w}, the quadrature weights.
}

\references{
  Berman, M. and Turner, T.R.
  Approximating point process likelihoods with GLIM.
  \emph{Applied Statistics} \bold{41} (1992) 31--38.
}
\author{Rolf Turner
  \email{r.turner@auckland.ac.nz}
}
\examples{
data(ogata)
deep    <- ogata$deep
shallow <- ogata$shallow
# Use preponly=TRUE to get the quadrature points.
D1 <- ppm1(shallow ~ .,Data=doLeg(5),qtype="q",stype=3,ndum=17,preponly=TRUE)
# Evaluate the effect of the deep quakes as predictors of the shallow quakes
# at each quadrature point.
M  <- outer(D1$qpoints,deep,"-")
D2 <- vector("list",3)
d  <- 2.06e-4
for(k in 1:3) {
   D2[[k]] <- apply(M^(k-1) * exp(-d*M),1,sum)
}
names(D2) <- paste("psi",1:3,sep="_")
# Fit the model to the "trend" given by Legendre polynomials
# and the effect of the deep quakes.
fit <- ppm1(shallow ~ .,Data=c(doLeg(5),D2),qtype="q",stype=3,ndum=17)
}

\keyword{ models }
